@@ -13,12 +13,12 @@ module Propellor.Property.Docker (
 	docked,
 	memoryLimited,
 	garbageCollected,
+	tweaked,
 	Image,
 	ContainerName,
 	-- * Container configuration
 	dns,
 	hostname,
-	name,
 	publish,
 	expose,
 	user,
@@ -29,6 +29,9 @@ module Propellor.Property.Docker (
 	cpuShares,
 	link,
 	ContainerAlias,
+	restartAlways,
+	restartOnFailure,
+	restartNever,
 	-- * Internal use
 	chain,
 ) where
@@ -102,10 +105,10 @@ docked hosts cn = RevertableProperty
   where
 	go desc a = property (desc ++ " " ++ cn) $ do
 		hn <- asks hostName
-  		let cid = ContainerId hn cn
+		let cid = ContainerId hn cn
 		ensureProperties [findContainer mhost cid cn $ a cid]
 		
-	mhost = findHost hosts (cn2hn cn)
+	mhost = findHostNoAlias hosts (cn2hn cn)
 
 	setup cid (Container image runparams) =
 		provisionContainer cid
@@ -149,14 +152,18 @@ findContainer mhost cid cn mk = case mhost of
 mkContainer :: ContainerId -> Host -> Maybe Container
 mkContainer cid@(ContainerId hn _cn) h = Container
 	<$> fromVal (_dockerImage info)
-	<*> pure (map (\a -> a hn) (_dockerRunParams info))
+	<*> pure (map (\mkparam -> mkparam hn) (_dockerRunParams info))
   where
 	info = _dockerinfo $ hostInfo h'
-  	h' = h
-		-- expose propellor directory inside the container
+	h' = h
+		-- Restart by default so container comes up on
+		-- boot or when docker is upgraded.
+		&^ restartAlways
+		-- Expose propellor directory inside the container.
 		& volume (localdir++":"++localdir)
-		-- name the container in a predictable way so we
-		-- and the user can easily find it later
+		-- Name the container in a predictable way so we
+		-- and the user can easily find it later. This property
+		-- comes last, so it cannot be overridden.
 		& name (fromContainerId cid)
 
 -- | Causes *any* docker images that are not in use by running containers to
@@ -176,6 +183,16 @@ garbageCollected = propertyList "docker garbage collected"
 	gcimages = property "docker images garbage collected" $ do
 		liftIO $ report <$> (mapM removeImage =<< listImages)
 
+-- | Tweaks a container to work well with docker.
+--
+-- Currently, this consists of making pam_loginuid lines optional in
+-- the pam config, to work around https://github.com/docker/docker/issues/5663
+-- which affects docker 1.2.0.
+tweaked :: Property
+tweaked = trivial $
+	cmdProperty "sh" ["-c", "sed -ri 's/^session\\s+required\\s+pam_loginuid.so$/session optional pam_loginuid.so/' /etc/pam.d/*"]
+	`describe` "tweaked for docker"
+
 -- | Configures the kernel to respect docker memory limits. 
 --
 -- This assumes the system boots using grub 2. And that you don't need any
@@ -206,7 +223,7 @@ dns = runProp "dns"
 hostname :: String -> Property
 hostname = runProp "hostname"
 
--- | Set name for container. (Normally done automatically.)
+-- | Set name of container.
 name :: String -> Property
 name = runProp "name"
 
@@ -263,6 +280,25 @@ link linkwith calias = genProp "link" $ \hn ->
 -- Each container has its own alias namespace.
 type ContainerAlias = String
 
+-- | This property is enabled by default for docker containers configured by
+-- propellor; as well as keeping badly behaved containers running,
+-- it ensures that containers get started back up after reboot or
+-- after docker is upgraded.
+restartAlways :: Property
+restartAlways = runProp "restart" "always"
+
+-- | Docker will restart the container if it exits nonzero.
+-- If a number is provided, it will be restarted only up to that many
+-- times.
+restartOnFailure :: Maybe Int -> Property
+restartOnFailure Nothing = runProp "restart" "on-failure"
+restartOnFailure (Just n) = runProp "restart" ("on-failure:" ++ show n)
+
+-- | Makes docker not restart a container when it exits
+-- Note that this includes not restarting it on boot!
+restartNever :: Property
+restartNever = runProp "restart" "no"
+
 -- | A container is identified by its name, and the host
 -- on which it's deployed.
 data ContainerId = ContainerId HostName ContainerName
@@ -303,30 +339,38 @@ runningContainer :: ContainerId -> Image -> [RunParam] -> Property
 runningContainer cid@(ContainerId hn cn) image runps = containerDesc cid $ property "running" $ do
 	l <- liftIO $ listContainers RunningContainers
 	if cid `elem` l
-		then do
-			-- Check if the ident has changed; if so the
-			-- parameters of the container differ and it must
-			-- be restarted.
-			runningident <- liftIO $ getrunningident
-			if runningident == Just ident
-				then noChange
-				else do
-					void $ liftIO $ stopContainer cid
-					restartcontainer
+		then checkident =<< liftIO (getrunningident simpleShClient)
 		else ifM (liftIO $ elem cid <$> listContainers AllContainers)
-			( restartcontainer
+			( do
+				-- The container exists, but is not
+				-- running. Its parameters may have
+				-- changed, but we cannot tell without
+				-- starting it up first.
+				void $ liftIO $ startContainer cid
+				-- It can take a while for the container to
+				-- start up enough to get its ident, so
+				-- retry for up to 60 seconds.
+				checkident =<< liftIO (getrunningident (simpleShClientRetry 60))
 			, go image
 			)
   where
 	ident = ContainerIdent image hn cn runps
 
+	-- Check if the ident has changed; if so the
+	-- parameters of the container differ and it must
+	-- be restarted.
+	checkident runningident
+		| runningident == Just ident = noChange
+		| otherwise = do
+			void $ liftIO $ stopContainer cid
+			restartcontainer
+
 	restartcontainer = do
 		oldimage <- liftIO $ fromMaybe image <$> commitContainer cid
 		void $ liftIO $ removeContainer cid
 		go oldimage
 
-	getrunningident :: IO (Maybe ContainerIdent)
-	getrunningident = simpleShClient (namedPipe cid) "cat" [propellorIdent] $ \rs -> do
+	getrunningident shclient = shclient (namedPipe cid) "cat" [propellorIdent] $ \rs -> do
 		let !v = extractident rs
 		return v
 
@@ -422,6 +466,9 @@ provisionContainer cid = containerDesc cid $ property "provisioned" $ liftIO $ d
 stopContainer :: ContainerId -> IO Bool
 stopContainer cid = boolSystem dockercmd [Param "stop", Param $ fromContainerId cid ]
 
+startContainer :: ContainerId -> IO Bool
+startContainer cid = boolSystem dockercmd [Param "start", Param $ fromContainerId cid ]
+
 stoppedContainer :: ContainerId -> Property
 stoppedContainer cid = containerDesc cid $ property desc $ 
 	ifM (liftIO $ elem cid <$> listContainers RunningContainers)
